#ifndef _dmsJPEGImage_h_
#define _dmsJPEGImage_h_

#include <GL/gl.h>
#include <dms/Image.h>

namespace dms
{

class JPEGImage : public Image
    {
    public:
        JPEGImage(char * filename=0);
        void loadFile(char * filename);
    private:
    };

}

#endif
