#include <GL/gl.h>
#include "dms/Transform.h"

namespace dms
{

Transform::Transform(void)
    {
    }


Transform::~Transform(void)
    {
    }


void Transform::pushApply(void)
    {
    glPushMatrix();
    apply();
    }


void Transform::pop(void)
    {
    glPopMatrix();
    }

}
