#ifndef _dmsFileImage_h_
#define _dmsFileImage_h_

#include <dms/Image.h>

namespace dms
{

class FileImage : public Image
    {
    public:
        FileImage(char * filename=0);
        void loadFile(char * filename);
    private:
        int fileType(char *filename);
        void loadTiffFile(char *filename);
        void loadJpegFile(char *filename);
        void loadSgiFile(char *filename);
    };

}

#endif
