#include "dms/Texture.h"

namespace dms
{

Texture::Texture(GLint wrap, GLint min, GLint mag)
    {
    setMinFilter(min);
    setMagFilter(mag);
    setWrap(wrap);
    }


Texture::~Texture(void)
    {
    }


void Texture::disable(void)
    {
    }


void Texture::setMinFilter(GLint v)
    {
    minFilter_ = v;
    }


void Texture::setMagFilter(GLint v)
    {
    magFilter_ = v;
    }


void Texture::setWrapS(GLint v)
    {
    wrapS_ = v;
    }


void Texture::setWrapT(GLint v)
    {
    wrapT_ = v;
    }


void Texture::setWrapR(GLint v)
    {
    wrapR_ = v;
    }


}
