#ifndef _dmsTransform_h_
#define _dmsTransform_h_

namespace dms
{

class Transform
    {
    public:
        Transform(void);
        virtual ~Transform(void);
        virtual void apply(void) = 0;
        void pushApply(void);
        void pop(void);
    private:
    };

}

#endif
