#ifndef _dmsImage_h_
#define _dmsImage_h_

#include <GL/gl.h>

namespace dms
{

class Image
    {
    public:
        Image(void);
        Image(GLsizei width, GLsizei height, void *data=0);
        ~Image(void);
        GLsizei width(void) const { return width_; }
        GLsizei height(void) const { return height_; }
        void *data(void) const { return imagedata_; }
        void setSize(GLsizei width, GLsizei height);
        void setData(void *);
        void allocateData(void);
        void freeData(void);
    private:
        GLsizei width_, height_;
        void * imagedata_;
        bool allocatedData_;
    };

}

#endif
