#ifndef _dmsTIFFImage_h_
#define _dmsTIFFImage_h_

#include <GL/gl.h>
#include <dms/Image.h>

namespace dms
{

class TIFFImage : public Image
    {
    public:
        TIFFImage(char * filename=0);
        void loadFile(char * filename);
    private:
    };

}

#endif
